﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ESG.Drivers.SscpClient;
using ESG.Drivers.SscpParser;

namespace ESG.SharkPLC.SscpDemo
{
	class Program
	{
		/// <summary>
		/// Variable definitions correspond to the demo project (NetSdkDemoPlcProject)
		/// Variables are addressed by an ID and offset and length of the buffer (each variable is trasmitted as its binary image).
		/// </summary>
		private static VariableDefinition[] m_Variables = new VariableDefinition[]
			{
				new VariableDefinition(1, 0, 4), // variable "a"
				new VariableDefinition(2, 0, 4), // variable "b"
				new VariableDefinition(3, 0, 4), // variable "c"
			};

		static void Main(string[] args)
		{
			using (SharkTcpClient client = new SharkTcpClient(512, TimeSpan.FromSeconds(10), TimeSpan.FromSeconds(10), ProtocolParser.ChecksumType.None))
			{
				Console.WriteLine("Connecting...");

				if (client.Connect(new TcpConnectionParameters("localhost", 12346, false)))
				{
					Console.WriteLine("Connected");

					Console.WriteLine("Sending login...");

					byte plcAddress = 1;

					// header - PLC address
					if (client.LoginExtended(plcAddress, "admin", "rw", String.Empty))
					{
						Console.WriteLine("Logged in");

						DateTime timestamp;
						IEnumerable<RawHiLoValue> values;

						Console.WriteLine("Reading data...");
						if (client.ReadVariables(plcAddress, ValueTransferFlags.GetAllValues, 0, m_Variables, out timestamp, out values))
						{
							Console.WriteLine("Data read:");

							foreach (var value in values)
							{
								Console.WriteLine("CommUid: {0} = {1}", value.VariableItem.Uid, BEConverter.GetFloat32FromBuffer(value.Data, 0));
							}
						}
						else
						{
							Console.WriteLine("Cannot read data");
						}
					}
					else
					{
						Console.WriteLine("Cannot log in");
					}

					if (!client.Disconnect())
					{
						Console.WriteLine("Cannot dicsonnect");
					}
				}
				else
				{
					Console.WriteLine("Cannot connect");
				}
			}
		}
	}
}
